#lecture5.py

# common import
import sys
sys.path.append('..')
from common.core import *
from common.kinect import *
from common.gfxutil import *

from kivy.uix.widget import Widget
from kivy.graphics import Color, Ellipse, Line
from kivy.graphics.instructions import InstructionGroup

import numpy as np

# get basic Kinect module working - showing data from right-hand position
class MainWidget1(BaseWidget) :
    def __init__(self):
        super(MainWidget1, self).__init__()

        self.label = topleft_label()
        self.add_widget(self.label)

        self.kinect = Kinect()
        self.kinect.add_joint(Kinect.kRightHand)

    def on_update(self) :
        self.kinect.on_update()
        pt = self.kinect.get_joint(Kinect.kRightHand)
        self.label.text = 'x:%d\ny:%d\nz:%d\n' % (pt[0], pt[1], pt[2])


# show a single hand graphically
class MainWidget2(BaseWidget) :
    def __init__(self):
        super(MainWidget2, self).__init__()

        self.label = topleft_label()
        self.add_widget(self.label)

        self.kinect = Kinect()
        self.kinect.add_joint(Kinect.kRightHand)

        self.right_hand = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.2, .6, .2))
        self.canvas.add(self.right_hand)

    def on_update(self) :
        self.kinect.on_update()

        pt = self.kinect.get_joint(Kinect.kRightHand)
        range_min = np.array([-250.0, -200, -500])
        range_max = np.array([700.0, 700, 0])
        norm_pt = scale_point(pt, range_min, range_max)

        self.label.text = 'x:%d   %.2f\ny:%d   %.2f\nz:%d   %.2f\n' %  (pt[0], norm_pt[0], pt[1], norm_pt[1], pt[2], norm_pt[2])

        self.right_hand.set_pos(norm_pt)


# Show 4 different skeletal joings - two hands, two knees
class MainWidget3(BaseWidget) :
    def __init__(self):
        super(MainWidget3, self).__init__()

        # and text to display our status
        self.label = topleft_label()
        self.add_widget(self.label)

        self.kinect = Kinect()
        self.kinect.add_joint(Kinect.kRightHand)
        self.kinect.add_joint(Kinect.kLeftHand)
        self.kinect.add_joint(Kinect.kRightKnee)
        self.kinect.add_joint(Kinect.kLeftKnee)


        self.right_hand = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.2, .6, .2))
        self.canvas.add(self.right_hand)
        self.left_hand  = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.2, .2, .6))
        self.canvas.add(self.left_hand)
        self.right_knee = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.7, .6, .2))
        self.canvas.add(self.right_knee)
        self.left_knee  = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.7, .2, .6))
        self.canvas.add(self.left_knee)


    def on_update(self) :
        self.kinect.on_update()

        right_hand_pt = self.kinect.get_joint(Kinect.kRightHand)
        left_hand_pt  = self.kinect.get_joint(Kinect.kLeftHand)
        right_knee_pt = self.kinect.get_joint(Kinect.kRightKnee)
        left_knee_pt  = self.kinect.get_joint(Kinect.kLeftKnee)

        # for scaling joint locations -> [0,1]
        # this scaling sets up a retangular box 2 meters wide, 2 meters tall, and 1 meter in depth
        # centered around the player's toroso
        range_min = np.array([-1000.0, -1000, -1000])
        range_max = np.array([1000.0, 1000, 0])

        right_hand_norm = scale_point(right_hand_pt, range_min, range_max)
        left_hand_norm  = scale_point(left_hand_pt, range_min, range_max)
        right_knee_norm = scale_point(right_knee_pt, range_min, range_max)
        left_knee_norm  = scale_point(left_knee_pt, range_min, range_max)

        self.label.text  = "RH: %.2f %.2f %.2f\n" % tuple(right_hand_norm.tolist())
        self.label.text += "LH: %.2f %.2f %.2f\n" % tuple(left_hand_norm.tolist())
        self.label.text += "RN: %.2f %.2f %.2f\n" % tuple(right_knee_norm.tolist())
        self.label.text += "LN: %.2f %.2f %.2f\n" % tuple(left_knee_norm.tolist())

        self.right_hand.set_pos(right_hand_norm)
        self.left_hand.set_pos(left_hand_norm)
        self.right_knee.set_pos(right_knee_norm)
        self.left_knee.set_pos(left_knee_norm)


# A clap gesture detector. Notifies a callback when a clap is detected.
# the callback function takes a single argument: pos (3D point)
class ClapGesture(object):
    def __init__(self, kinect, callback):
        super(ClapGesture, self).__init__()
        self.kinect = kinect
        self.callback = callback

        self.clap_thresh = 100
        self.unclap_thresh = 200

        self.is_clapped = False

    def on_update(self, left_pt, right_pt):

        dist = np.linalg.norm( left_pt - right_pt )
        if dist < self.clap_thresh and not self.is_clapped:
            print "clap!!"

            mid_point = (left_pt + right_pt) / 2

            self.callback(mid_point)

            self.is_clapped = True

        if dist > self.unclap_thresh and self.is_clapped:
            self.is_clapped = False
            print "UN clap!!"



# Demonstrate the Clap Gesture: Tracks the 2 hands and displays the Pop objects
# when the hands clap
class MainWidget4(BaseWidget) :
    def __init__(self):
        super(MainWidget4, self).__init__()

        # and text to display our status
        self.label = topleft_label()
        self.add_widget(self.label)

        self.kinect = Kinect()
        self.kinect.add_joint(Kinect.kRightHand)
        self.kinect.add_joint(Kinect.kLeftHand)

        self.right_hand = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.2, .6, .2))
        self.canvas.add(self.right_hand)
        self.left_hand  = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.2, .2, .6))
        self.canvas.add(self.left_hand)


        self.anims = AnimGroup()
        self.canvas.add(self.anims)

        self.clap = ClapGesture(self.kinect, self.on_clap)

    # scale a point from millimeters to [0-1] space
    def _scale_point(self, pt) :
        # for scaling hand locations -> [0,1]
        # this scaling sets up a retangular box 2 meters wide, 2 meters tall, and 1 meter in depth
        # centered around the player's toroso
        range_min = np.array([-1000.0, -1000, -1000])
        range_max = np.array([1000.0, 1000, 0])
        return scale_point(pt, range_min, range_max)

    def on_update(self) :
        self.kinect.on_update()

        # get points (in mm)
        right_hand_pt = self.kinect.get_joint(Kinect.kRightHand)
        left_hand_pt = self.kinect.get_joint(Kinect.kLeftHand)

        # feed data to clap gesture detector
        self.clap.on_update(left_hand_pt, right_hand_pt)

        # normalize pts for display
        right_hand_norm = self._scale_point(right_hand_pt)
        left_hand_norm = self._scale_point(left_hand_pt)

        self.label.text  = "RH: %.2f %.2f %.2f\n" % tuple(right_hand_norm.tolist())
        self.label.text += "LH: %.2f %.2f %.2f\n" % tuple(left_hand_norm.tolist())

        self.right_hand.set_pos(right_hand_norm)
        self.left_hand.set_pos(left_hand_norm)

        self.anims.on_update()

    # callback happens when a clap is detected.
    def on_clap(self, pos):
        # pos is in millimeters. Normalize (and only keep x and y):
        norm_pos = self._scale_point(pos)[0:2]

        # convert to screen coordinates for display
        screen_pos = norm_pos * np.array([Window.width/2,Window.height/2]) + np.array([10,10])

        # create a graphic that shows the clap
        pop = Pop(screen_pos)
        self.anims.add(pop)



# An animation / display to go along with the Clap Gesture
class Pop(InstructionGroup) :
    def __init__(self, pos):
        super(Pop, self).__init__()

        self.color = Color(1,0,0,1)
        self.add(self.color)

        self.circle = CEllipse(cpos = pos)
        self.add(self.circle)

        # create keyframe animation: a frame is: (time, alpha radius)
        self.anim = KFAnim((0,   1, 30),
                           (0.5, 0, 60),)

        self.time = 0
        self.on_update(0)

    def on_update(self, dt):
        values = self.anim.eval(self.time)
        self.color.a = values[0]
        self.circle.csize = values[1] * 2, values[1] * 2
        self.time += dt

        return self.anim.is_active(self.time)



# convert pt into unit scale (ie, range [0,1]) assuming that pt falls in the
# the range [min_val, max_val]
# value is clipped [0,1]
def scale_point(pt, min_val, max_val):
    pt = (pt - min_val) / (max_val - min_val)
    pt = np.clip(pt, 0, 1)
    return pt


# pass in which MainWidget to run as a command-line arg
run(eval('MainWidget' + sys.argv[1]))
